@@ -1,6 +1,7 @@
 /*
- 	Ray
-    Copyright (C) 2010, 2011, 2012  Sébastien Boisvert
+    Ray -- Parallel genome assemblies for parallel DNA sequencing
+    Copyright (C) 2010, 2011, 2012, 2013 Sébastien Boisvert
+    Copyright (C) 2013 Charles Joly Beauparlant
 
 	http://DeNovoAssembler.SourceForge.Net/
 
@@ -14,38 +15,41 @@
     GNU General Public License for more details.
 
     You have received a copy of the GNU General Public License
-    along with this program (gpl-3.0.txt).  
+    along with this program (gpl-3.0.txt)
 	see <http://www.gnu.org/licenses/>
 */
 
 // TODO -- there should not be any plugin construction in this file
 
-#include <plugin_VerticesExtractor/GridTableIterator.h>
-#include <cryptography/crypto.h>
-#include <core/OperatingSystem.h>
-#include <structures/SplayNode.h>
-#include <application_core/Machine.h>
-#include <core/statistics.h>
-#include <plugin_VerticesExtractor/VerticesExtractor.h>
-#include <profiling/Profiler.h>
-#include <sstream>
-#include <communication/Message.h>
-#include <time.h>
-#include <plugin_SeedExtender/TipWatchdog.h>
-#include <plugin_SeedExtender/BubbleTool.h>
-#include <assert.h>
-#include <application_core/common_functions.h>
+#include "Machine.h"
+
+#include <code/VerticesExtractor/GridTableIterator.h>
+#include <code/VerticesExtractor/VerticesExtractor.h>
+#include <code/SeedExtender/TipWatchdog.h>
+#include <code/SeedExtender/BubbleTool.h>
+#include <code/Mock/common_functions.h>
+#include <code/Mock/constants.h>
+#include <code/CoverageGatherer/CoverageDistribution.h>
+#include <code/SequencesLoader/Read.h>
+#include <code/SequencesLoader/Loader.h>
+
+#include <RayPlatform/profiling/Profiler.h>
+#include <RayPlatform/communication/Message.h>
+#include <RayPlatform/structures/SplayTreeIterator.h>
+#include <RayPlatform/structures/SplayNode.h>
+#include <RayPlatform/core/statistics.h>
+#include <RayPlatform/core/OperatingSystem.h>
+#include <RayPlatform/memory/MyAllocator.h>
+#include <RayPlatform/cryptography/crypto.h>
+#include <RayPlatform/core/types.h>
+
 #include <iostream>
 #include <fstream>
-#include <plugin_CoverageGatherer/CoverageDistribution.h>
 #include <string.h>
-#include <structures/SplayTreeIterator.h>
-#include <plugin_SequencesLoader/Read.h>
-#include <plugin_SequencesLoader/Loader.h>
-#include <memory/MyAllocator.h>
-#include <application_core/constants.h>
+#include <assert.h>
+#include <time.h>
 #include <algorithm>
-
+#include <sstream>
 using namespace std;
 
 /* Pick-up the option -show-communication-events */
@@ -55,14 +59,17 @@ using namespace std;
  */
 Machine::Machine(int argc,char**argv){
 
-	void constructor(int*argc,char**argv);
+	m_argc=argc;
+	m_argv=argv;
 
-	m_computeCore.constructor(&argc,&argv);
-	m_messagesHandler=m_computeCore.getMessagesHandler();
+}
 
+void Machine::init(int argc,char**argv){
+
+	m_rank=m_computeCore.getRank();
+	m_size=m_computeCore.getSize();
+	
 	m_alive=m_computeCore.getLife();
-	m_rank=m_messagesHandler->getRank();
-	m_size=m_messagesHandler->getSize();
 
 	m_inbox=m_computeCore.getInbox();
 	m_outbox=m_computeCore.getOutbox();
@@ -87,21 +94,25 @@ Machine::Machine(int argc,char**argv){
 		if(param.find("help")!=string::npos){
 			if(isMaster())
 				m_parameters.showUsage();
+
 			m_computeCore.destructor();
 			m_aborted=true;
 		}else if(param.find("usage")!=string::npos){
 			if(isMaster())
 				m_parameters.showUsage();
+
 			m_computeCore.destructor();
 			m_aborted=true;
 		}else if(param.find("man")!=string::npos){
 			if(isMaster())
 				m_parameters.showUsage();
+
 			m_computeCore.destructor();
 			m_aborted=true;
 		}else if(param.find("-version")!=string::npos){
 			if(isMaster())
 				showRayVersionShort();
+
 			m_computeCore.destructor();
 			m_aborted=true;
 		}
@@ -110,7 +121,9 @@ Machine::Machine(int argc,char**argv){
 	if(m_aborted)
 		return;
 
-	cout<<"Rank "<<m_rank<<": Rank= "<<m_rank<<" Size= "<<m_size<<" ProcessIdentifier= "<<portableProcessId()<<" ProcessorName= "<<*(m_messagesHandler->getName())<<endl;
+	#if 0
+	" ProcessorName= "<<*(m_messagesHandler->getName())<<endl;
+	#endif
 
 	m_argc=argc;
 	m_argv=argv;
@@ -129,7 +142,7 @@ Machine::Machine(int argc,char**argv){
 	m_helper.constructor(argc,argv,&m_parameters,m_switchMan,m_outboxAllocator,m_outbox,&m_aborted,
 		&m_coverageDistribution,&m_numberOfMachinesDoneSendingCoverage,&m_numberOfRanksWithCoverageData,
 	&m_reductionOccured,m_ed,m_fusionData,m_profiler,&m_networkTest,m_seedingData,
-	&m_timePrinter,&m_seedExtender,&m_scaffolder,m_messagesHandler,m_inbox,
+	&m_timePrinter,&m_seedExtender,&m_scaffolder,m_inbox,
 	&m_oa,&m_isFinalFusion,m_bubbleData,m_alive,
 	&m_CLEAR_n,&m_DISTRIBUTE_n,&m_FINISH_n,&m_searcher,
 	&m_numberOfRanksDoneSeeding,&m_numberOfRanksDoneDetectingDistances,&m_numberOfRanksDoneSendingDistances,
@@ -154,12 +167,13 @@ m_virtualCommunicator,&m_kmerAcademyBuilder,
  */
 void Machine::start(){
 
+	init(m_argc,m_argv);
+
+
 	if(m_aborted)
 		return;
 
-	m_partitioner.constructor(m_outboxAllocator,m_inbox,m_outbox,&m_parameters,m_switchMan);
-
-	// TODO: move this in plugin Searcher 
+	// TODO: move this in plugin Searcher
 	m_searcher.constructor(&m_parameters,m_outbox,&m_timePrinter,m_switchMan,m_virtualCommunicator,m_inbox,
 		m_outboxAllocator,&m_subgraph);
 
@@ -176,7 +190,6 @@ void Machine::start(){
 	m_numberOfMachinesReadyForEdgesDistribution=0;
 	m_aborted=false;
 	m_readyToSeed=0;
-	m_wordSize=-1;
 	m_last_value=0;
 	m_mode_send_ingoing_edges=false;
 	m_mode_send_vertices=false;
@@ -191,37 +204,41 @@ void Machine::start(){
 	m_sequence_ready_machines=0;
 	m_isFinalFusion=false;
 
-	m_messagesHandler->barrier();
 
 	if(isMaster()){
 		cout<<endl<<"**************************************************"<<endl;
     		cout<<"This program comes with ABSOLUTELY NO WARRANTY."<<endl;
     		cout<<"This is free software, and you are welcome to redistribute it"<<endl;
-    		cout<<"under certain conditions; see \"LICENSE\" for details."<<endl;
+		cout<<"under certain conditions; see \"LICENSE.txt\" for details."<<endl;
 		cout<<"**************************************************"<<endl;
 		cout<<endl;
-		cout<<"Ray Copyright (C) 2010, 2011, 2012  Sébastien Boisvert"<<endl;
+		cout<<"Ray Copyright (C) 2010, 2011, 2012, 2013 Sébastien Boisvert"<<endl;
 		cout<<"Centre de recherche en infectiologie de l'Université Laval"<<endl;
 		cout<<"Project funded by the Canadian Institutes of Health Research (Doctoral award 200902CGM-204212-172830 to S.B.)"<<endl;
  		cout<<"http://denovoassembler.sf.net/"<<endl<<endl;
 
-		cout<<"Reference to cite: "<<endl<<endl;
+		cout<<"References to cite: "<<endl<<endl;
+
+		cout << "Sébastien Boisvert, Frédéric Raymond, Élénie Godzaridis, François Laviolette and Jacques Corbeil." <<endl;
+		cout << "Ray Meta: scalable de novo metagenome assembly and profiling."<<endl;
+		cout<<"Genome Biology (BioMed Central Ltd)."<<endl;
+		cout<<"13:R122, Published: 22 December 2012"<<endl;
+		cout<<"http://dx.doi.org/doi:10.1186/gb-2012-13-12-r122"<<endl;
+		cout << endl;
+
 		cout<<"Sébastien Boisvert, François Laviolette & Jacques Corbeil."<<endl;
 		cout<<"Ray: simultaneous assembly of reads from a mix of high-throughput sequencing technologies."<<endl;
 		cout<<"Journal of Computational Biology (Mary Ann Liebert, Inc. publishers, New York, U.S.A.)."<<endl;
 		cout<<"November 2010, Volume 17, Issue 11, Pages 1519-1533."<<endl;
-		cout<<"doi:10.1089/cmb.2009.0238"<<endl;
 		cout<<"http://dx.doi.org/doi:10.1089/cmb.2009.0238"<<endl;
 		cout<<endl;
 	}
 
-	m_messagesHandler->barrier();
-
 	m_parameters.setSize(getSize());
 
 	// this peak is attained in VerticesExtractor::deleteVertices
 	// 2012-03-20: which peaks ?
-
+	// 2012-11-06: probably memory usage (Virtual Memory)
 
 	/**
  * 		build the plugins now */
@@ -235,7 +252,6 @@ void Machine::start(){
 	m_coverageGatherer.constructor(&m_parameters,m_inbox,m_outbox,m_switchMan->getSlaveModePointer(),&m_subgraph,
 		m_outboxAllocator);
 
-
 	m_fusionTaskCreator.constructor(m_virtualProcessor,m_outbox,
 		m_outboxAllocator,m_switchMan->getSlaveModePointer(),&m_parameters,&(m_ed->m_EXTENSION_contigs),
 		&(m_ed->m_EXTENSION_identifiers),&(m_fusionData->m_FUSION_eliminated),
@@ -246,7 +262,6 @@ void Machine::start(){
 		&(m_ed->m_EXTENSION_identifiers),&(m_fusionData->m_FUSION_eliminated),
 		m_virtualCommunicator,&(m_fusionData->m_FINISH_newFusions));
 
-
 	m_amos.constructor(&m_parameters,m_outboxAllocator,m_outbox,m_fusionData,m_ed,m_switchMan->getMasterModePointer(),m_switchMan->getSlaveModePointer(),&m_scaffolder,
 		m_inbox,m_virtualCommunicator);
 
@@ -259,15 +274,13 @@ void Machine::start(){
 
 	m_ranksDoneAttachingReads=0;
 
-	m_messagesHandler->barrier();
-
 	// TODO: check if 65536 is really a limit.
 	// the limit is probably in a header somewhere in application_core or in RayPlatform.
 	// with routing enabled, the limit is 4096 compute cores.
 	
 	int maximumNumberOfProcesses=65536;
 	if(getSize()>maximumNumberOfProcesses){
-		cout<<"The maximum number of processes is "<<maximumNumberOfProcesses<<" (this can be changed in the code)"<<endl;
+		cout<<"Error: The maximum number of processes is "<<maximumNumberOfProcesses<<" (this can be changed in the code)"<<endl;
 	}
 
 	assert(getSize()<=maximumNumberOfProcesses);
@@ -281,32 +294,44 @@ void Machine::start(){
 	}
 
 	if(isMaster()){
-		showRayVersion(m_messagesHandler,fullReport);
+		showRayVersion(fullReport);
 
 		if(!fullReport){
 			cout<<endl;
 		}
 	}
 
+	int miniRanksPerRank=m_computeCore.getMiniRanksPerRank();
+
 	/** only show the version. */
 	if(fullReport){
 		m_computeCore.destructor();
 		return;
 	}
 
-	m_parameters.constructor(m_argc,m_argv,getRank(),m_size);
+	m_parameters.constructor(m_argc,m_argv,getRank(),m_size,miniRanksPerRank);
 
-	if(m_parameters.runProfiler())
+	m_partitioner.constructor(m_outboxAllocator,m_inbox,m_outbox,&m_parameters,m_switchMan);
+
+	if(m_parameters.runProfiler()) {
 		m_computeCore.enableProfiler();
+	}
 
 	if(m_parameters.hasOption("-with-profiler-details"))
 		m_computeCore.enableProfilerVerbosity();
 
+	if(m_parameters.hasOption("-debug")) {
+		m_computeCore.enableProfiler();
+		m_computeCore.enableProfilerVerbosity();
+	}
+
 	if(m_parameters.showCommunicationEvents())
 		m_computeCore.showCommunicationEvents();
 	
+	string*name=m_computeCore.getMessagesHandler()->getName();
+
 	// initiate the network test.
-	m_networkTest.constructor(m_rank,m_size,m_inbox,m_outbox,&m_parameters,m_outboxAllocator,m_messagesHandler->getName(),
+	m_networkTest.constructor(m_rank,m_size,m_inbox,m_outbox,&m_parameters,m_outboxAllocator,name,
 		&m_timePrinter);
 
 	m_networkTest.setSwitchMan(m_switchMan);
@@ -315,48 +340,82 @@ void Machine::start(){
 	m_persistentAllocator.constructor(PERSISTENT_ALLOCATOR_CHUNK_SIZE,"RAY_MALLOC_TYPE_PERSISTENT_DATA_ALLOCATOR",
 		m_parameters.showMemoryAllocations());
 
-	int directionAllocatorChunkSize=4194304; // 4 MiB
+	// options are loaded from here
+	// plus the directory exists now
+	if(m_parameters.hasOption("-route-messages")){
 
-	m_directionsAllocator.constructor(directionAllocatorChunkSize,"RAY_MALLOC_TYPE_WAVE_ALLOCATOR",
-		m_parameters.showMemoryAllocations());
+		ostringstream routingPrefix;
+		routingPrefix<<m_parameters.getPrefix()<<"/Routing/";
 
-	/** create the directory for the assembly */
-	
-	string directory=m_parameters.getPrefix();
-	if(fileExists(directory.c_str())){
-		if(m_parameters.getRank() == MASTER_RANK)
-			cout<<"Error, "<<directory<<" already exists, change the -o parameter to another value."<<endl;
+		m_router->enable(m_inbox,m_outbox,m_outboxAllocator,m_parameters.getRank(),
+			routingPrefix.str(),m_parameters.getSize(),
+			m_parameters.getConnectionType(),m_parameters.getRoutingDegree());
 
-		m_computeCore.destructor();
-		return;
+		// update the connections
+		vector<int> connections;
+		m_router->getGraph()->getIncomingConnections(m_parameters.getRank(),&connections);
+		
 	}
 
-	m_messagesHandler->barrier();
+	if(m_parameters.hasOption("-verify-message-integrity")) {
+		m_computeCore.enableCheckSums();
+	}
+
+/*
+ * We need to know if we must abort. All the RayPlatform
+ * engine will start anyway on each core, so we need to stop
+ * them if necessary.
+ */
+	bool oldDirectoryExists=false;
+
+	string directory=m_parameters.getPrefix();
+	if(fileExists(directory.c_str())){
+		if(m_parameters.getRank() == MASTER_RANK){
+			cout<<"Error, "<<directory<<" already exists, change the -o parameter to another value."<<endl;
+			cout<<endl;
+
+/*
+ * Tell the first plugin that the job will not run.
+ */
+			m_helper.notifyThatOldDirectoryExists();
+
+			oldDirectoryExists=true;
+		}
+	}
 	
 	// create the directory
-	if(m_parameters.getRank() == MASTER_RANK){
+	/** create the directory for the assembly */
+	if(!oldDirectoryExists && m_parameters.getRank() == MASTER_RANK){
 		createDirectory(directory.c_str());
 		m_parameters.writeCommandFile();
 
 		m_timePrinter.setFile(m_parameters.getPrefix());
-	}
 
-	cout<<endl;
+/*
+ * Write routing information only if the directory is sane.
+ */
+		if(m_router->isEnabled())
+			m_router->writeFiles();
+
+	}
 
 	// register the plugins.
 	registerPlugins();
 
-	cout<<endl;
-	
 	// write a report about plugins
-	if(m_parameters.getRank()==MASTER_RANK){
-		ostringstream directory;
-		directory<<m_parameters.getPrefix()<<"/Plugins";
+	if(!oldDirectoryExists && m_parameters.getRank()==MASTER_RANK){
 
-		string file=directory.str();
-		createDirectory(file.c_str());
+		bool mustWritePluginData = m_parameters.hasOption("-write-plugin-data");
+
+		if(mustWritePluginData) {
+			ostringstream directory;
+			directory<<m_parameters.getPrefix()<<"/Plugins";
+
+			string file=directory.str();
+			createDirectory(file.c_str());
 	
-		m_computeCore.printPlugins(file);
+			m_computeCore.printPlugins(file);
+		}
 
 		// write the version of RayPlatform
 		
@@ -368,28 +427,8 @@ void Machine::start(){
 		f7.close();
 	}
 
-	ostringstream routingPrefix;
-	routingPrefix<<m_parameters.getPrefix()<<"/Routing/";
-
-	// options are loaded from here
-	// plus the directory exists now
-	if(m_parameters.hasOption("-route-messages")){
-
-		if(m_parameters.getRank()== MASTER_RANK)
-			createDirectory(routingPrefix.str().c_str());
-
-		m_router->enable(m_inbox,m_outbox,m_outboxAllocator,m_parameters.getRank(),
-			routingPrefix.str(),m_parameters.getSize(),
-			m_parameters.getConnectionType(),m_parameters.getRoutingDegree());
-
-		// update the connections
-		vector<int> connections;
-		m_router->getGraph()->getIncomingConnections(m_parameters.getRank(),&connections);
-		m_messagesHandler->setConnections(&connections);
-	}
-
 	// set the attributes of the seed extender.
-	m_seedExtender.constructor(&m_parameters,&m_directionsAllocator,m_ed,&m_subgraph,m_inbox,m_profiler,
+	m_seedExtender.constructor(&m_parameters,m_ed,&m_subgraph,m_inbox,m_profiler,
 		m_outbox,m_seedingData,m_switchMan->getSlaveModePointer(),&(m_seedingData->m_SEEDING_vertexCoverageRequested),
 		&(m_seedingData->m_SEEDING_vertexCoverageReceived),m_outboxAllocator,m_fusionData,
 		&(m_seedingData->m_SEEDING_seeds),m_bubbleData,&(m_seedingData->m_SEEDING_edgesRequested),
@@ -397,15 +436,12 @@ void Machine::start(){
 		&(m_seedingData->m_SEEDING_currentVertex),&(m_seedingData->m_SEEDING_receivedVertexCoverage),
 		&(m_seedingData->m_SEEDING_receivedOutgoingEdges),&m_c,&m_oa);
 
-
 	m_kmerAcademyBuilder.constructor(m_parameters.getSize(),&m_parameters,&m_subgraph,
 		&m_myReads,m_inbox,m_outbox,m_switchMan->getSlaveModePointer(),m_outboxAllocator);
 
 	m_si.constructor(&m_parameters,m_outboxAllocator,m_inbox,m_outbox,m_virtualCommunicator,
 		m_switchMan->getSlaveModePointer(),&m_myReads);
 
-
-
 	m_profiler = m_computeCore.getProfiler();
 	m_profiler->constructor(m_parameters.runProfiler());
 
@@ -438,24 +474,18 @@ void Machine::start(){
 	&m_parameters,m_seedingData,m_inbox,m_virtualCommunicator);
 
 	m_subgraph.constructor(getRank(),&m_parameters);
+
+	if(!oldDirectoryExists)
+		m_subgraph.printStatus();
 	
-	m_seedingData->constructor(&m_seedExtender,getRank(),getSize(),m_outbox,m_outboxAllocator,m_switchMan->getSlaveModePointer(),&m_parameters,&m_wordSize,&m_subgraph,m_inbox,m_virtualCommunicator);
+	m_seedingData->constructor(&m_seedExtender,getRank(),getSize(),m_outbox,m_outboxAllocator,m_switchMan->getSlaveModePointer(),&m_parameters,&m_subgraph,m_inbox,m_virtualCommunicator);
 
 	(*m_alive)=true;
 	m_totalLetters=0;
 
-	m_messagesHandler->barrier();
-
-	if(m_parameters.showMemoryUsage()){
-		showMemoryUsage(getRank());
-	}
-
-	m_messagesHandler->barrier();
-
-	if(isMaster()){
-		cout<<endl;
-	}
-
+/*
+ * Build the monstruous object.
+ */
 	m_mp.constructor(
 m_router,
 m_seedingData,
@@ -471,7 +501,6 @@ m_seedingData,
 			getRank(),
 			&m_numberOfMachinesDoneSendingEdges,
 			m_fusionData,
-			&m_wordSize,
 			&m_myReads,
 		getSize(),
 	m_inboxAllocator,
@@ -489,7 +518,6 @@ m_seedingData,
 	&m_readyToSeed,
 	&m_FINISH_n,
 	&m_reductionOccured,
-	&m_directionsAllocator,
 	&m_mode_send_coverage_iterator,
 	&m_coverageDistribution,
 	&m_sequence_ready_machines,
@@ -505,6 +533,23 @@ m_seedingData,
 	&m_numberOfRanksWithCoverageData,&m_seedExtender,
 	m_switchMan->getMasterModePointer(),&m_isFinalFusion,&m_si);
 
+	bool mustWriteSchedulingInformation = m_parameters.hasOption("-write-scheduling-data");
+
+/*
+ * Create the Scheduling before calling run().
+ * run() is like a barrier.
+ */
+	// log ticks
+	if(!oldDirectoryExists && m_parameters.getRank()==MASTER_RANK
+			&& mustWriteSchedulingInformation){
+
+		ostringstream scheduling;
+		scheduling<<m_parameters.getPrefix()<<"/Scheduling/";
+		createDirectory(scheduling.str().c_str());
+	}
+
+	cout<<"Rank "<<m_rank<<": Rank= "<<m_rank<<" Size= "<<m_size<<" ProcessIdentifier= "<<portableProcessId()<<endl;
+
 	if(m_argc==1||((string)m_argv[1])=="--help"){
 		if(isMaster()){
 			m_aborted=true;
@@ -515,68 +560,46 @@ m_seedingData,
 		m_computeCore.run();
 	}
 
-	if(isMaster() && !m_aborted){
+	/*if(isMaster() && !m_aborted){
 		m_timePrinter.printDurations();
-
 		cout<<endl;
-	}
-
-	m_messagesHandler->barrier();
-
-	if(m_parameters.showMemoryUsage()){
-		showMemoryUsage(getRank());
-	}
-
-	m_messagesHandler->barrier();
+	}*/
 
+/*
+ * TODO: the code above should go in a plugin, not here.
+ */
 	if(isMaster() && !m_aborted && !m_parameters.hasOption("-test-network-only")){
 		m_scaffolder.printFinalMessage();
-		cout<<endl;
-		cout<<"Rank "<<getRank()<<" wrote "<<m_parameters.getOutputFile()<<endl;
-		cout<<"Rank "<<getRank()<<" wrote "<<m_parameters.getScaffoldFile()<<endl;
-		cout<<"Check for "<<m_parameters.getPrefix()<<"*"<<endl;
-		cout<<endl;
-		if(m_parameters.useAmos()){
-			cout<<"Rank "<<getRank()<<" wrote "<<m_parameters.getAmosFile()<<" (reads mapped onto contiguous sequences in AMOS format)"<<endl;
-
-		}
-		cout<<endl;
 	}
 
-	m_messagesHandler->barrier();
+	if(!oldDirectoryExists && !m_aborted
+			&& mustWriteSchedulingInformation){
 
+		ostringstream masterTicks;
+		masterTicks<<m_parameters.getPrefix()<<"/Scheduling/"<<m_parameters.getRank()<<".MasterTicks.txt";
+		ofstream f1(masterTicks.str().c_str());
+		m_tickLogger->printMasterTicks(&f1);
+		f1.close();
 
-	// log ticks
-	if(m_parameters.getRank()==MASTER_RANK){
-		ostringstream scheduling;
-		scheduling<<m_parameters.getPrefix()<<"/Scheduling/";
-		createDirectory(scheduling.str().c_str());
+		ostringstream slaveTicks;
+		slaveTicks<<m_parameters.getPrefix()<<"/Scheduling/"<<m_parameters.getRank()<<".SlaveTicks.txt";
+		ofstream f2(slaveTicks.str().c_str());
+		m_tickLogger->printSlaveTicks(&f2);
+		f2.close();
 	}
 
-	// wait for master to create the directory.
-	m_messagesHandler->barrier();
-	
-	ostringstream masterTicks;
-	masterTicks<<m_parameters.getPrefix()<<"/Scheduling/"<<m_parameters.getRank()<<".MasterTicks.txt";
-	ofstream f1(masterTicks.str().c_str());
-	m_tickLogger->printMasterTicks(&f1);
-	f1.close();
-
-	ostringstream slaveTicks;
-	slaveTicks<<m_parameters.getPrefix()<<"/Scheduling/"<<m_parameters.getRank()<<".SlaveTicks.txt";
-	ofstream f2(slaveTicks.str().c_str());
-	m_tickLogger->printSlaveTicks(&f2);
-	f2.close();
-
-
-	m_messagesHandler->freeLeftovers();
 	m_persistentAllocator.clear();
-	m_directionsAllocator.clear();
-	m_inboxAllocator->clear();
-	m_outboxAllocator->clear();
 
 	m_diskAllocator.clear();
 
+/*
+ * The app is responsible for destroying the core.
+ * It should work also without this call because it
+ * is RayPlatform that destroys the MessagesHandler.
+ */
+
+// TODO: it would be nice to have a callback to deregister plugins
+
 	m_computeCore.destructor();
 }
 
@@ -599,7 +622,7 @@ Machine::~Machine(){
 
 
 void Machine::showRayVersionShort(){
-	cout<<"Ray version "<<RAY_VERSION<<endl;
+	cout<<"Ray version "<<CONFIG_RAY_VERSION<<endl;
 	cout<<"License for Ray: GNU General Public License version 3"<<endl;
 
 	cout<<"RayPlatform version: "<<m_computeCore.getRayPlatformVersion()<<endl;
@@ -609,14 +632,13 @@ void Machine::showRayVersionShort(){
 	CoverageDepth maximumCoverageDepth=0;
 	maximumCoverageDepth--; // underflow it
 
-	cout<<"MAXKMERLENGTH: "<<MAXKMERLENGTH<<endl;
-	cout<<"KMER_U64_ARRAY_SIZE: "<<KMER_U64_ARRAY_SIZE<<endl;
+	cout<<"MAXKMERLENGTH: "<<CONFIG_MAXKMERLENGTH<<endl;
 	cout<<"Maximum coverage depth stored by CoverageDepth: "<<maximumCoverageDepth<<endl;
 	cout<<"MAXIMUM_MESSAGE_SIZE_IN_BYTES: "<<MAXIMUM_MESSAGE_SIZE_IN_BYTES<<" bytes"<<endl;
 
 /*
 content
-	cout<<"option"; 
+	cout<<"option";
 	#ifdef option
 	cout<<"y";
 	#else
@@ -625,16 +647,16 @@ content
 	cout<<endl;
 
 list
- FORCE_PACKING 
- ASSERT 
- HAVE_LIBZ 
- HAVE_LIBBZ2 
- CONFIG_PROFILER_COLLECT 
- CONFIG_CLOCK_GETTIME 
- __linux__ 
- _MSC_VER 
- __GNUC__ 
- RAY_32_BITS 
+ FORCE_PACKING
+ ASSERT
+ CONFIG_HAVE_LIBZ
+ HAVE_LIBBZ2
+ CONFIG_PROFILER_COLLECT
+ CONFIG_CLOCK_GETTIME
+ __linux__
+ _MSC_VER
+ __GNUC__
+ RAY_32_BITS
  RAY_64_BITS
 
 for i in $(cat list ); do exp="s/option/$i/g"; sed $exp content; done > list2
@@ -643,7 +665,7 @@ for i in $(cat list ); do exp="s/option/$i/g"; sed $exp content; done > list2
 	/* generated code */
 
 	cout<<"FORCE_PACKING = ";
-	#ifdef FORCE_PACKING
+	#ifdef CONFIG_FORCE_PACKING
 	cout<<"y";
 	#else
 	cout<<"n";
@@ -659,7 +681,7 @@ for i in $(cat list ); do exp="s/option/$i/g"; sed $exp content; done > list2
 	cout<<endl;
 
 	cout<<"HAVE_LIBZ = ";
-	#ifdef HAVE_LIBZ
+	#ifdef CONFIG_HAVE_LIBZ
 	cout<<"y";
 	#else
 	cout<<"n";
@@ -667,14 +689,14 @@ for i in $(cat list ); do exp="s/option/$i/g"; sed $exp content; done > list2
 	cout<<endl;
 
 	cout<<"HAVE_LIBBZ2 = ";
-	#ifdef HAVE_LIBBZ2
+	#ifdef CONFIG_HAVE_LIBBZ2
 	cout<<"y";
 	#else
 	cout<<"n";
 	#endif
 	cout<<endl;
 
-	cout<<"CONFIG_PROFILER_COLLECT = ";
+	cout<<"PROFILER_COLLECT = ";
 	#ifdef CONFIG_PROFILER_COLLECT
 	cout<<"y";
 	#else
@@ -682,7 +704,15 @@ for i in $(cat list ); do exp="s/option/$i/g"; sed $exp content; done > list2
 	#endif
 	cout<<endl;
 
-	cout<<"CONFIG_CLOCK_GETTIME = ";
+	cout<<"MINIRANKS = ";
+#ifdef CONFIG_MINI_RANKS
+	cout<<"y";
+#else
+	cout<<"n";
+#endif
+	cout<<endl;
+
+	cout<<"CLOCK_GETTIME = ";
 	#ifdef CONFIG_CLOCK_GETTIME
 	cout<<"y";
 	#else
@@ -749,6 +779,17 @@ for i in $(cat list ); do exp="s/option/$i/g"; sed $exp content; done > list2
 	cout<<"Compiler: GNU gcc/g++ "<<__VERSION__<<endl;
 	#endif
 
+/**
+ * Report availability of MPI I/O
+ */
+	cout<<"MPI I/O: ";
+
+#ifdef CONFIG_MPI_IO
+	cout<<"y";
+#else
+	cout<<"n";
+#endif
+	cout<<endl;
 	
 	#ifdef __SSE4_2__
 	cout<<"With SSE 4.2"<<endl;
@@ -759,39 +800,9 @@ for i in $(cat list ); do exp="s/option/$i/g"; sed $exp content; done > list2
 	#endif
 }
 
-void Machine::showRayVersion(MessagesHandler*messagesHandler,bool fullReport){
+void Machine::showRayVersion(bool fullReport){
 	showRayVersionShort();
 
-	cout<<endl;
-	cout<<"Rank "<<MASTER_RANK<<": Operating System: ";
-	cout<<getOperatingSystem()<<endl;
-
-
-	cout<<"Message-passing interface"<<endl;
-	cout<<endl;
-	cout<<"Rank "<<MASTER_RANK<<": Message-Passing Interface implementation: ";
-	cout<<messagesHandler->getMessagePassingInterfaceImplementation()<<endl;
-
-	int version;
-	int subversion;
-	messagesHandler->version(&version,&subversion);
-
-	cout<<"Rank "<<MASTER_RANK<<": Message-Passing Interface standard version: "<<version<<"."<<subversion<<""<<endl;
-
-
-	cout<<endl;
-
-	if(!fullReport)
-		return;
-
-	cout<<endl;
-
-	cout<<"Number of MPI ranks: "<<messagesHandler->getSize()<<endl;
-	cout<<"Ray master MPI rank: "<<MASTER_RANK<<endl;
-	cout<<"Ray slave MPI ranks: 0-"<<messagesHandler->getSize()-1<<endl;
-	cout<<endl;
-
-	cout<<endl;
 }
 
 void Machine::registerPlugins(){
@@ -818,6 +829,10 @@ void Machine::registerPlugins(){
 	m_computeCore.registerPlugin(&m_phylogeny);
 	m_computeCore.registerPlugin(&m_genomeNeighbourhood);
 	m_computeCore.registerPlugin(&m_ontologyPlugin);
+	m_computeCore.registerPlugin(&m_mock);
+	m_computeCore.registerPlugin(&m_example);
+	m_computeCore.registerPlugin(&m_pathEvaluator);
+	m_computeCore.registerPlugin(&m_spuriousSeedAnnihilator);
 
 	// resolve the symbols
 	// this is done here because we want to write a summary for
@@ -825,3 +840,12 @@ void Machine::registerPlugins(){
 	// otherwise, symbols are resolved when ComputeCore::run() is called.
 	m_computeCore.resolveSymbols();
 }
+
+void Machine::run(){
+
+	start();
+}
+
+
+
+
